'use strict';

var postcss = require('postcss');

var _require = require('./keyframes'),
    isKeyframeRule = _require.isKeyframeRule,
    isKeyframeAlreadyProcessed = _require.isKeyframeAlreadyProcessed,
    isKeyframeSymmetric = _require.isKeyframeSymmetric,
    rtlifyKeyframe = _require.rtlifyKeyframe;

var _require2 = require('./rules'),
    getDirRule = _require2.getDirRule,
    processSrcRule = _require2.processSrcRule;

var _require3 = require('./decls'),
    rtlifyDecl = _require3.rtlifyDecl,
    ltrifyDecl = _require3.ltrifyDecl;

module.exports = postcss.plugin('postcss-rtl', function () {
    return function (css) {

        // selectors have direction related properties
        // should add [dir] prefix to increase priority
        var dirRegExp = [/direction/im, /left/im, /right/im, /^(margin|padding|border-(color|style|width))$/ig, /border-radius/ig, /shadow/ig, /transform-origin/ig, /^(?!text\-).*?transform$/ig, /transition(-property)?$/i, /background(-position(-x)?|-image)?$/i, /float|clear|text-align/i, /cursor/i];

        var keyframes = [];

        // collect @keyframes
        css.walkAtRules(function (rule) {
            if (!isKeyframeRule(rule)) return;
            if (isKeyframeAlreadyProcessed(rule)) return;
            if (isKeyframeSymmetric(rule)) return;

            keyframes.push(rule.params);
            rtlifyKeyframe(rule);
        });

        // Simple rules (includes rules inside @media-queries)
        css.walkRules(function (rule) {
            var ltrDecls = [];
            var rtlDecls = [];
            var dirDecls = [];

            if (rule.selector.match(/\[dir/)) return;
            if (isKeyframeRule(rule.parent)) return;

            rule.walkDecls(function (decl) {
                var rtl = rtlifyDecl(decl, keyframes);

                if (rtl) {
                    ltrDecls.push(ltrifyDecl(decl, keyframes));
                    rtlDecls.push(decl.clone(rtl));
                    return;
                }

                if (dirRegExp.some(function (re) {
                    return !!decl.prop.match(re);
                })) {
                    dirDecls.push(decl);
                    decl.remove();
                }
            });

            if (rtlDecls.length) {
                (function () {
                    var ltrDirRule = void 0;
                    getDirRule(rule, 'rtl').append(rtlDecls);
                    ltrDirRule = getDirRule(rule, 'ltr');
                    ltrDecls.forEach(function (_decl) {
                        return _decl.moveTo(ltrDirRule);
                    });
                })();
            }

            if (dirDecls.length) {
                getDirRule(rule, 'dir').append(dirDecls);
            }

            /* set dir attrs */
            processSrcRule(rule);
        });
        return false;
    };
});